<?php
	// CERTIFICATE action handler
	// (c) 2023-2024 Frédéric Blondiau, DouWère s.p.r.l.

	error_reporting(E_ALL);
	ini_set('display_errors', 1);

	$DEBUG=1;

	require_once("logger.php");
	require_once("certificates/index.php");

	// https://www.uuidgenerator.net/dev-corner/php
	function uuid($data = null) {
		// Generate 16 bytes (128 bits) of random data or use the data passed into the function.
		$data = $data ?? random_bytes(16);
		assert(strlen($data) == 16);
		
		// Set version to 0100
		$data[6] = chr(ord($data[6]) & 0x0f | 0x40);
		// Set bits 6-7 to 10
		$data[8] = chr(ord($data[8]) & 0x3f | 0x80);
		
		// Output the 36 character UUID.
		return strtoupper(vsprintf('%s%s-%s-%s-%s-%s%s%s', str_split(bin2hex($data), 4)));
	}

	function userEmail($installationID) {
		$dbFolder = getcwd() . '/../../../db/';
		$db = new PDO("sqlite:" . $dbFolder . "logger.sqlite");
		try
		{
			$stm = $db -> prepare('SELECT json_extract(params, "$.email") AS "email"
				FROM "logs" 
				WHERE "action" = "login" AND
				json_extract(params, "$.installationID") == :installationID
				ORDER BY "rowID" DESC LIMIT 1'
			);
			$stm -> execute([
				':installationID' => $installationID
			]);
			if ($row = $stm->fetch(PDO::FETCH_ASSOC)) {
				return $row["email"];
			} else {
				return "";
			}
		}
		catch(PDOException $e)
		{
			printf("prepare select failed " . ($e->getMessage()) . " \n");
			return "";
		}
	}

	function handle($params) {
		if (isset($params["installationID"]) && isset($params["testDisplayName"]) && isset($params["testComment"]) && 
				isset($params["environment"]) && isset($params["result"])) {
			$installationID = $params["installationID"];

			$row["title"] = "BIO K " . substr($params["result"]["qrCode"], 7, 3); // $params["testDisplayName"];
			$row["testComment"] = $params["testComment"];
			$row["installationID"] = $installationID;

			$row["milliStamp"] = $params["environment"]["milliStamp"];
			$row["device"] = $params["environment"]["machine"] . " (" . $params["environment"]["os"] . ")";
			$row["mode"] = $params["environment"]["mode"];

			$row["QR"] = $params["result"]["qrCode"];
			$row["result"] = $params["result"]["result"] . ".";
			$row["location"] = $params["result"]["location"];
			$row["image"] = $params["result"]["image"];

			$row["documentID"] = uuid();
			$row["timestamp"] = date("Y-m-d H:i:s", intval($row["milliStamp"] / 1000));
			$row["batch"] = substr($row["QR"], 2, 5);
			$row["tracking"] = substr($row["QR"], -12);
			$row["email"] = userEmail($installationID);

			$doneInfo = array();

			if ($row["email"] != "") {
				if ($row["mode"] == 'activated') {
					// print_r($row);
					$doneInfo = handleCertificate($row);
				} else {
					$doneInfo["notActivated"] = true;
				}
			} else {
				$doneInfo["noEmail"] = true;
			}

			$loggerReply = logger("certificate", $params);
			if (isset($loggerReply["result"])) {
				$loggerReply["result"] = array_merge($loggerReply["result"], $doneInfo);
			}
			return $loggerReply;
		} else {
			return array (
				"error" => array (
					"code" => -1,
					"message" => "generic error"
				)
			);
		}
	}
?>
