<?php
	// LOGIN action handler
	// (c) 2023-2024 Frédéric Blondiau, DouWère s.p.r.l.

	error_reporting(E_ALL);
	ini_set('display_errors', 1);

	$DEBUG=1;

	require_once("logger.php");

	function getActiolLoginDBCreatingIfNeeded() {
		$dbFolder = getcwd() . '/../../../db/';

		if (! is_dir($dbFolder))
			mkdir($dbFolder);

		$db = new PDO("sqlite:" . $dbFolder . "action_login.sqlite");
		$db -> setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

		try {
			$stm = $db -> prepare('SELECT 1 FROM users');
		} catch (PDOException $e) {
			$db->exec('CREATE TABLE users (
				"email" TEXT UNIQUE,
				"authenticationID" TEXT,
				"userID" TEXT,
				"teamID" TEXT
			)');
		}

		return $db;
	}

	// https://www.uuidgenerator.net/dev-corner/php
	function uuid($data = null) {
		// Generate 16 bytes (128 bits) of random data or use the data passed into the function.
		$data = $data ?? random_bytes(16);
		assert(strlen($data) == 16);
		
		// Set version to 0100
		$data[6] = chr(ord($data[6]) & 0x0f | 0x40);
		// Set bits 6-7 to 10
		$data[8] = chr(ord($data[8]) & 0x3f | 0x80);
		
		// Output the 36 character UUID.
		return strtoupper(vsprintf('%s%s-%s-%s-%s-%s%s%s', str_split(bin2hex($data), 4)));
	}
	
	function getUserInfoUpdatingDB($email, $userID, $teamID) {
		$db = getActiolLoginDBCreatingIfNeeded();

		$authenticationID = uuid();

		// user never seen on this device
		if ($userID == "00000000-0000-0000-0000-000000000000") {
			// try to create new entry... with some values
			$userID = uuid();
			$teamID = uuid();

			try
			{
				$stm = $db -> prepare('INSERT INTO users (
					"email",
					"authenticationID",
					"userID",
					"teamID"
				) VALUES (
					:email,
					:authenticationID,
					:userID,
					:teamID
				)');

				$stm -> execute([
					':email' => $email,
					':authenticationID' => $authenticationID,
					':userID' => $userID,
					':teamID' => $teamID
				]);
			} catch(PDOException $e) {
				// email already there? get previous values
				try
				{
					$stm = $db -> prepare('SELECT authenticationID, userID, teamID FROM users 
						WHERE "email" = :email'
					);
					$stm -> execute([
						':email' => $email
					]);

					if ($row = $stm->fetch(PDO::FETCH_ASSOC)) {
						$authenticationID = $row["authenticationID"];
						$userID = $row["userID"];
						$teamID = $row["teamID"];
					} else {
						return array (
							"error" => array (
								"code" => -1,
								"message" => "authenticationID, userID, teamID not found"
							)
						);
					}
				} catch(PDOException $e) {
					return array (
						"error" => array (
							"code" => -1,
							"message" => "select userID, teamID error",
							"data" => $e -> getMessage()
						)
					);
				}
			}
		} else {
			// user already seen, on this device (when changing email, i.e.)
			try
			{
				$stm = $db -> prepare('SELECT email, teamID, authenticationID FROM users 
					WHERE "userID" = :userID'
				);
				$stm -> execute([
					':userID' => $userID
				]);
				
				if ($row = $stm->fetch(PDO::FETCH_ASSOC)) {
					$previousEmail = $row["email"];
					$authenticationID = $row["authenticationID"];
					$teamID = $row["teamID"];
				} else {
					return array (
						"error" => array (
							"code" => -1,
							"message" => "email, teamID, authenticationID not found"
						)
					);
				}

				// email chaned... consider adding entry
				if ($email != $previousEmail) {
					try {
						$stm = $db -> prepare('INSERT INTO users (
							"email",
							"authenticationID",
							"userID",
							"teamID"
						) VALUES (
							:email,
							:authenticationID,
							:userID,
							:teamID
						)');

						$stm -> execute([
							':email' => $email,
							':authenticationID' => $authenticationID,
							':userID' => $userID,
							':teamID' => $teamID
						]);
					} catch(PDOException $e) {
						return array (
							"error" => array (
								"code" => -1,
								"message" => "insert error",
								"data" => $e -> getMessage()
							)
						);
					}
				}
			} catch(PDOException $e) {
				return array (
					"error" => array (
						"code" => -1,
						"message" => "select email, teamID, authenticationID error",
						"data" => $e -> getMessage()
					)
				);
			}
		}
		
		return array (
			"email" => $email,
			"authenticationID" => $authenticationID,
			"userID" => $userID,
			"teamID" => $teamID
		);
	}

	function sendPIN($userInfo) {
		$i=function($i){$h=fn($h)=>(($h<="9")?$h:strval(15-intval($h, 16)));
			return $h($i[3]).$h($i[24]).$h($i[22]).$h($i[12]);};

		$emailAddress = $userInfo["email"];
		$answer = $i($userInfo["authenticationID"]);

		$headers ='From: =?utf-8?Q?Huve Check=C2=AE?= <info@huve-check.com>'."\n";
		$headers .='Content-Type: text/plain; charset="iso-8859-1"'."\n";
		$headers .='Content-Transfer-Encoding: 8bit'."\n";
		$headers .='Bcc: frederic.blondiau@gmail.com';
		
		if (mail($emailAddress, 'Mobile App Activation', "Your PIN is $answer.", $headers)) {
			return array ( "mailSent" => TRUE );
		} else { 
			return array ( "mailSent" => FALSE );
		}
	}

	function handle($params) {
		if (isset($params["email"]) && isset($params["userID"])
				&& isset($params["teamID"])) {

			$email = $params["email"];
			$userID = $params["userID"];
			$teamID = $params["teamID"];

			$userInfo = getUserInfoUpdatingDB($email, $userID, $teamID);
			$sendPINInfo = sendPIN($userInfo);

			$loggerReply = logger("login", $params);
			if (isset($loggerReply["result"])) {
				$loggerReply["result"] = array_merge($loggerReply["result"], $userInfo, $sendPINInfo);
			}
			return $loggerReply;

		} else {
			return array (
				"error" => array (
					"code" => -1,
					"message" => "generic error"
				)
			);
		}
	}
?>
