<?php
	// Certificate generator and sender
	// (c) 2023-2024 Frédéric Blondiau, DouWère s.p.r.l.

	use setasign\Fpdi\Fpdi;
	use PHPMailer\PHPMailer\PHPMailer;
	use PHPMailer\PHPMailer\Exception;

	require_once('phpqrcode/qrlib.php');
	require_once('fpdf/fpdf.php');
	require_once('fpdi2/src/autoload.php');

	require_once('PHPMailer/src/Exception.php');
	require_once('PHPMailer/src/PHPMailer.php');
	require_once('PHPMailer/src/SMTP.php');

	error_reporting(E_ALL|E_STRICT);
	ini_set('display_errors', 1);

	$DEBUG=1;

	// 0.1 second
	usleep(1000000);
	date_default_timezone_set("Europe/Brussels");

	$lastSeenStamp = time();
	$humanReadable=date('r', $lastSeenStamp);

	class PDF_DouWere extends FPDi
	{
		var $angle=0;

		function Rotate($angle,$x=-1,$y=-1)
		{
			if($x==-1)
				$x=$this->x;
			if($y==-1)
				$y=$this->y;
			if($this->angle!=0)
				$this->_out('Q');
			$this->angle=$angle;
			if ($angle!=0) {
				$angle*=M_PI/180;
				$c=cos($angle);
				$s=sin($angle);
				$cx=$x*$this->k;
				$cy=($this->h-$y)*$this->k;
				$this->_out(sprintf('q %.5F %.5F %.5F %.5F %.2F %.2F cm 1 0 0 1 %.2F %.2F cm',$c,$s,-$s,$c,$cx,$cy,-$cx,-$cy));
			}
		}

		function _endpage()
		{
			if ($this->angle!=0)
				{
					$this->angle=0;
					$this->_out('Q');
				}
			parent::_endpage();
		}

		function RotatedText($x,$y,$txt,$angle)
		{
			//Text rotated around its origin
			$this->Rotate($angle,$x,$y);
			$this->Text($x,$y,$txt);
			$this->Rotate(0);
		}

		function RotatedImage($file,$x,$y,$w,$h,$angle)
		{
			//Image rotated around its upper-left corner
			$this->Rotate($angle,$x,$y);
			$this->Image($file,$x,$y,$w,$h);
			$this->Rotate(0);
		}

		protected $extgstates = array();

		// alpha: real value from 0 (transparent) to 1 (opaque)
		// bm:    blend mode, one of the following:
		//          Normal, Multiply, Screen, Overlay, Darken, Lighten, ColorDodge, ColorBurn,
		//          HardLight, SoftLight, Difference, Exclusion, Hue, Saturation, Color, Luminosity
		function SetAlpha($alpha, $bm='Normal')
		{
			// set alpha for stroking (CA) and non-stroking (ca) operations
			$gs = $this->AddExtGState(array('ca'=>$alpha, 'CA'=>$alpha, 'BM'=>'/'.$bm));
			$this->SetExtGState($gs);
		}

		function AddExtGState($parms)
		{
			$n = count($this->extgstates)+1;
			$this->extgstates[$n]['parms'] = $parms;
			return $n;
		}

		function SetExtGState($gs)
		{
			$this->_out(sprintf('/GS%d gs', $gs));
		}

		function _enddoc()
		{
			if(!empty($this->extgstates) && $this->PDFVersion<'1.4')
				$this->PDFVersion='1.4';
			parent::_enddoc();
		}

		function _putextgstates()
		{
			for ($i = 1; $i <= count($this->extgstates); $i++)
			{
				$this->_newobj();
				$this->extgstates[$i]['n'] = $this->n;
				$this->_put('<</Type /ExtGState');
				$parms = $this->extgstates[$i]['parms'];
				$this->_put(sprintf('/ca %.3F', $parms['ca']));
				$this->_put(sprintf('/CA %.3F', $parms['CA']));
				$this->_put('/BM '.$parms['BM']);
				$this->_put('>>');
				$this->_put('endobj');
			}
		}

		function _putresourcedict()
		{
			parent::_putresourcedict();
			$this->_put('/ExtGState <<');
			foreach($this->extgstates as $k=>$extgstate)
				$this->_put('/GS'.$k.' '.$extgstate['n'].' 0 R');
			$this->_put('>>');
		}

		function _putresources()
		{
			$this->_putextgstates();
			parent::_putresources();
		}
	}

	function isLearningTest($scannedValue) {
		return $scannedValue[1] == "Y";
	}

	function createCertificatePDF($row) {
		global $pdf;

		// values from the placeholdersPositionInPDF tool
		$keysAndXY = array (
//			"QR" => array (512.53, 266.74),
			"QR" => array (512.53, 225.46),
			"result" => array (123.24, 287.87),
			"email" => array (30.39, 332.05),
			"device" => array (62.78, 355.09),
			"location" => array (70.25, 365.96),
			"timestamp" => array (82.47, 377.11),
			"installationID" => array (94.17, 388.16),
			"documentID" => array (92.39, 399.04),
			"image" => array (314.12, 378.53),
			"title" => array (518.63, 377.11),
			"batch" => array (546.60, 388.15),
			"tracking" => array (498.16, 399.05)
		);

		// initiate FPDI
		$pdf = new PDF_DouWere();

//        $pdf->AddFont('RobotoSlab-Medium','','RobotoSlab-Medium.php');
//        $pdf->SetFont('RobotoSlab-Medium');

		$pdf->AddPage('L', 'A5');

		$pdf->setSourceFile('certificates/certificate-en.pdf');
		$tplIdx = $pdf->importPage(1);
		$pdf->useTemplate($tplIdx, 0, 0, 210.1);

		$pdf->SetFont('Courier');

		foreach ($keysAndXY as $key => $value) {
			// formatting
			switch ($key) {
				case "result":
					$pdf->SetFontSize(16);
					if (mb_strtolower($row[$key][0]) == "p") {
						$pdf->SetTextColor(255, 0, 0);
					} else if (mb_strtolower($row[$key][0]) == "n") {
						$pdf->SetTextColor(1, 155, 62);
					} else {
						$pdf->SetTextColor(255, 147, 0);
					}
					$pdf->SetAlpha(1);
					$xOffset = 0;
					$yOffset = -1.1;
					break;
				case "QR":
//					$pdf->SetFontSize(19);
					$pdf->SetFontSize(15.2);
					$pdf->SetTextColor(255, 0xA5, 0);
					$pdf->SetAlpha(1);
					$xOffset = 0;
					$yOffset = 5;
					break;
				case "image":
					$xOffset = 22.44;
					$yOffset = -24.22;
					$pdf->SetAlpha(1);
					break;
				case "scannedValue":
					$pdf->SetFontSize(11.5);
					$pdf->SetTextColor(0);
					$pdf->SetAlpha(0.25);
					$xOffset = -36;
					$yOffset = -0.8;
					break;
				default:
					$pdf->SetFontSize(11.5);
					$pdf->SetTextColor(0);
					$pdf->SetAlpha(0.25);
					$xOffset = 0;
					$yOffset = -0.8;
			}

			// usage
			switch (substr($key, -2)) {
				case "QR":
					$tempFile = sys_get_temp_dir() . DIRECTORY_SEPARATOR . uniqid() . ".png";
					QRCode::png("https://b.rpdt.be/" . $row[$key], $tempFile);
//					$size = 40;
					$size = 32;
					$pdf->RotatedImage($tempFile,
						$keysAndXY["QR"][0] * 0.352778 + $size * sqrt(2) / 2,
						$keysAndXY["QR"][1] * 0.352778 - $size * sqrt(2) / 2,
						$size,
						$size,
						-135);
					unlink($tempFile);
					
					if (isLearningTest($row[$key])) {
						$pdf->RotatedText(
							$keysAndXY["QR"][0] * 0.352778 - $size * sqrt(2) / 2 + $xOffset,
							$keysAndXY["QR"][1] * 0.352778 - $size * sqrt(2) / 2 + $yOffset,
							"LEARNING",
							-45);
					}
					break;
				case "ge": // for image */
					$fileName = uniqid();
					
					$tempFile = sys_get_temp_dir() . DIRECTORY_SEPARATOR . $fileName . ".jpg";
					$decoded = base64_decode(str_replace(" ", "+", $row[$key]), true);
					
					if ($decoded) {
						file_put_contents($tempFile, $decoded);
						$finfo = finfo_open(FILEINFO_MIME_TYPE); // return mime type aka mimetype extension
						$mimeType = finfo_file($finfo, $tempFile);
//						print_r($mimeType);
						if (strcmp($mimeType, "image/x-portable-greymap") == 0) {
						// needs to change format from .pgm to .jpg
							$image = new Imagick($tempFile);
							$image->setImageFormat('jpeg');
							file_put_contents($tempFile, $image->getImageBlob());
						} else {
							// it's already a .jpg
						}
					} else {
						copy("dummy-image.jpg", $tempFile);
					}

					$pdf->RotatedImage($tempFile,
						$keysAndXY["image"][0] * 0.352778 + $xOffset,
						$keysAndXY["image"][1] * 0.352778 + $yOffset,
						30.5,
						7.7,
						-135);

					unlink($tempFile);
					break;
				case "ID":
					$pdf->Text(
						$value[0] * 0.352778 + $xOffset, $value[1] * 0.352778 + $yOffset,
						iconv('utf8', "cp1252//IGNORE", substr($row[$key], 0, 8))
					);
					break;
				default:
					$pdf->Text(
						$value[0] * 0.352778 + $xOffset, $value[1] * 0.352778 + $yOffset,
						iconv('utf8', "cp1252//IGNORE", $row[$key])
					);
			}
		}

		$pdf->SetTitle('Box', true);
		$pdf->SetAuthor('HuvePharma', true);

		$pdf->SetCreator('DouWère, s.p.r.l.', true);

		$pdf->Output('F', '../../../db/cache/' . $row["documentID"] . '.pdf');

		return;
	}

	function shouldSendCertificate($row) {
		global $lastSeenStamp, $humanReadable;

		// QR field contains scannedValue
		if (isLearningTest($row["QR"])) {
			// Learning tests certificates won't be sent twice to same user
			$userEmail = $row["email"];
		} else {
			// Other certificates won't be sent twice at all
			$userEmail = "";
		}

		$dbFolder = getcwd() . '/../../../db/';

		if (! is_dir($dbFolder))
			mkdir($dbFolder);

		$db = new PDO("sqlite:" . $dbFolder . "logger.sqlite");
		$db -> setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

		try {
			$stm = $db -> prepare('INSERT INTO certificates (
				"installationID",
				"documentID",
				"QR",
				"email",

				"humanReadable",
				"stamp"
			) VALUES (
				:installationID,
				:documentID,
				:QR,
				:email,

				:humanReadable,
				:stamp
			)');
			$stm -> execute([
				':installationID' => $row["installationID"],
				':documentID' => $row["documentID"],
				':QR' => $row["QR"],
				':email' => $userEmail,

				':humanReadable' => $humanReadable,
				':stamp' => $lastSeenStamp
			]);
			return true;
		}
		catch (PDOException $e) {
			return false;
		}
	}

	function handleCertificate($row) {
		$reply = array ();

		$email = new PHPMailer();
		$email->CharSet = "UTF-8";
		$email->SetFrom('info@huve-check.com', 'Huve Check®');
		$email->Subject = 'Huve Check Certificate';

		$email->AddAddress($row["email"]);
		$email->AddBcc("frederic.blondiau@gmail.com", 'Frédéric Blondiau');

		$isLearningTest = isLearningTest($row["QR"]);

		if ($isLearningTest || shouldSendCertificate($row)) {
			$email->Body = "Please, find here the certificate of your last analysis.";

			$file_to_attach = "../../../db/cache/" . $row["documentID"] . ".pdf";

			createCertificatePDF($row);
			$email->AddAttachment( $file_to_attach , 'Certificate.pdf' );
		} else {
			$email->Body = "The certificate for this analysis has already been sent.";
		}

		$reply["emailSent"] = $email->Send();

		return $reply;
	}
?>
