<?php
	// Generic JSON RPC handler
	// (c) 2023-2024 Frédéric Blondiau, DouWère s.p.r.l.

	error_reporting(E_ALL);
	ini_set('display_errors', 1);

	$DEBUG=1;

	sleep(1);

	$rawInput = file_get_contents('php://input');

	function L($string)
	{
		if (function_exists('L_locale'))
			return L_locale($string);
		else
			return trim(str_replace("_", " ", strtolower($string)));
	}

	function reply($reply) {
		// send reply as JSON encoded data

		header('Content-type: application/json');
		printf ("%s\n", json_encode($reply, JSON_FORCE_OBJECT));
		exit;
	}

	$query = json_decode($rawInput, true);
	$id = null;

	if (json_last_error() == JSON_ERROR_NONE) {

		$reply = array();

		$stamp=time();

		date_default_timezone_set("Europe/Brussels");
		$humanReadable=date('r', $stamp);

		$invitationDate=date("Y-m-d H:i", $stamp);

		try {
			if (!isset($query['jsonrpc'])) {
				throw new RuntimeException(L("_NOT_JSON_RPC_"));
			} else {
				$jsonrpc = $query['jsonrpc'];
				if ($jsonrpc == "2.0") {
					$reply['jsonrpc'] = $jsonrpc;
				} else {
					throw new RuntimeException(L("_NOT_JSONRPC_2_"));
				}
			}

			if (!isset($query['method'])) {
				throw new RuntimeException(L("_JSON_RPC_MISSING_METHOD_"));
			} else {
				$method = $query['method'];
			}

			if (!isset($query['params'])) {
				$params = null;
			} else {
				$params = $query['params'];
			}

			if (isset($query['id'])) {
				$id = intval($query['id']);
				if ($id != 0) {
					$reply['id'] = $id;
				} else {
					throw new RuntimeException(L("_JSON_RPC_BAD_ID_"));
				}
			}

			switch ($method) {
				case 'install':
				case 'start':
				case 'update':
				case 'picture':
				case 'result':
				case 'delete':
				case 'expired':
				case 'check':
				case 'done':
				case 'reset':
					require_once("logger.php");
					$reply = array_merge($reply, logger($method, $params));
					break;
				case 'login':
				case 'certificate':
					require_once("action_$method.php");
					$reply = array_merge($reply, handle($params));
					break;
				default:
					$reply ["error"] = array (
						"code" => -32601,
						"message"=> "Method not found"
					);
			}
		} catch (RuntimeException $e) {
			// trick to force sending answer
			$id = "null";
			$reply ["error"] = array (
				"code" => -32600,
				"message"=> "Invalid Request",
				"data" => $e -> getMessage()
			);
		}
	} else {
		// trick to force sending answer
		$id = "null";
		$reply ["error"] = array (
			"code" => -32700,
			"message"=> "Parse error"
		);
	}

	if (!is_null($id)) {
		reply($reply);
	}
?>
