<?php
	// Generic logger
	// (c) 2023-2024 Frédéric Blondiau, DouWère s.p.r.l.

	error_reporting(E_ALL);
	ini_set('display_errors', 1);

	$DEBUG=1;

	function getLoggerDBCreatingIfNeeded() {
		$dbFolder = getcwd() . '/../../../db/';

		if (! is_dir($dbFolder))
			mkdir($dbFolder);

		$db = new PDO("sqlite:" . $dbFolder . "logger.sqlite");
		$db -> setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

		try {
			$stm = $db -> prepare('SELECT 1 FROM logs');
		} catch (PDOException $e) {
			$db->exec('CREATE TABLE logs (
				"action" TEXT,
				"stamp" REAL,
				"date" TEXT,

				"params" TEXT
			)');
		}

		return $db;
	}

	function addToLogger($action, $stamp, $date, $params) {
		$db = getLoggerDBCreatingIfNeeded();
		try
		{
			$stm = $db -> prepare('INSERT INTO logs (
				"action",
				"stamp",
				"date",

				"params"
			) VALUES (
				:action,
				:stamp,
				:date,
			
				:params
			)');

			$stm -> execute([
				':action' => $action,
				':stamp' => $stamp,
				':date' => $date,

				':params' => json_encode($params)
			]);
			
			return "ok";
		} catch(PDOException $e) {
			return $e->getMessage();
		}
	}

	function logger($action, $params) {
		date_default_timezone_set("Europe/Brussels");

		$stamp = time();
		$date = date('r', $stamp);

		$loggerMessage = addToLogger($action, $stamp, $date, $params);
		if ($loggerMessage == "ok") {
			return array (
				"result" => array (
					"action" => $action,
					"stamp" => $stamp,
					"date" => $date
				)
			);
		} else {
			return array (
				"error" => array (
					"code" => -2,
					"message" => "can’t add to database",
					"data" => $loggerMessage
				)
			);
		}
	}
?>
